@@ -12,14 +12,9 @@ Graph::~Graph()
 {
 }
 /********************************************************************************************/
-double Graph::Dist(int *i, int *j)
-{
-    return this->pp->getDist(i, j);
-}
-/********************************************************************************************/
-/********************************************************************************************/
-void Graph::Init(Pp *pp0, int *gtype0, double *par0, double *prepR0, int *doDists0, int *toroidal0, int *dbg0)
+void Graph::Init(Pp *pp0, int *gtype0, double *par0, double *prepR0, int *doDists0, double *preDists, int *toroidal0, int *inc0, int *dbg0)
 {
+	int typein,i,j;
 	if(*dbg0)printf("intializing graph-object... ");
 
 	pp = pp0;
@@ -27,22 +22,38 @@ void Graph::Init(Pp *pp0, int *gtype0, double *par0, double *prepR0, int *doDist
 	prepR=prepR0;
 	opar = *par;
 	oldpar = &opar;
-	doDists=doDists0;dbg=dbg0;
-	toroidal= toroidal0;
-	nodelist.resize(*pp->n);
-	nodelistp = &nodelist;
+	doDists=doDists0;
+	dbg=dbg0;
+	inc = inc0;
+	nodelist.resize(pp->size());
 	gtype = gtype0;
+	mdeg = 0.0;
+	pp->setToroidal(toroidal0);
+	typeIncluded.clear();
+	for(i=0; i< pp->getNtypes(); i++)//need to ignore some types based on the inclusion vector
+	{
+		typein=0;
+		for(j=0;j< pp->size(); j++)
+			if( (pp->getT(&j) == pp->getTypevec(&i)) & inc[j])
+			{
+				typein=1;
+				break;
+			}
+		typeIncluded.push_back(typein);
+	}
 
-	if(toroidal)pp->toggleToroidal();
-	if(*doDists)  // distance triangle
+	if(preDists[0]>=0)
+	{
+		if(*dbg)printf("Setting precalculated distances...");
+		pp->setDists(preDists);
+		if(*dbg)printf("ok. ");
+	}
+	else if(*doDists)  // precalculate distance triangle
 	{
 		if(*dbg)printf("Precalculating distances...");
 		pp->calcDists();
 		if(*dbg)printf("ok. ");
 	}
-	pdone = 0;
-	prepDone = &pdone;
-
 	if(*dbg)printf(" done.\n");
 
 }
@@ -58,19 +69,14 @@ void Graph::setNodelist(std::vector<std::vector<int> > *nodelist_new)
 /********************************************************************************************/
 void Graph::setNodelist(SEXP prepGraph)
 {
-
-	int i;
 	nodelist.clear();nodelist.resize(0);
 	VectsxpToVector(getListElement(prepGraph,"edges"), nodelist);
 }
-
-
 /********************************************************************************************/
 SEXP Graph::toSEXP()
 //transform a std::vector<std::vector<int> > to SEXP, desctructive
 {
 	SEXP graph, *node;
-
 	PROTECT(graph = allocVector(VECSXP, this->nodelist.size()));
 	int i,j, *p, n;
 	for(i=0;i< (int) this->nodelist.size();i++)
@@ -94,7 +100,7 @@ SEXP Graph::toSEXP()
 }
 /********************************************************************************************/
 void Graph::remove_duplicates()
-// remove duplicates in edge lists, especially the simple delauney method creates them
+// remove duplicates in edge lists, especially the simple delaunay method creates them
 {
 	int i,j,k,isnew;
 	std::vector<int> *node;
@@ -142,12 +148,11 @@ void Graph::addNew(int i, int j)
 void Graph::sg_calc()
 {
 	// preprocess if requested
-	if( *prepR > 0 & *prepDone == 0 )
+	if(prepR[0]>0 && *oldpar<= *par )
 	{
 		if(*dbg)printf("Preprocessing[");
 		this->sg_geometric(prepR);
 		if(*dbg)printf("] ok.\n ");
-		*prepDone = 1;
 	}
 	//start the calculation
 	if(*gtype==0) //geometric
@@ -166,13 +171,14 @@ void Graph::sg_calc()
 	}
 	else if(*gtype==2) this->sg_mass_geometric();
 	else if(*gtype==3) this->sg_gabriel();
-	else if(*gtype==4) this->sg_delauney();
+	else if(*gtype==4) this->sg_delaunay();
 	else if(*gtype==5) this->sg_MST();
 	else if(*gtype==6) this->sg_markcross();
 	else if(*gtype==7) this->sg_SIG();
 	else if(*gtype==8) this->sg_RST();
 	else if(*gtype==9) this->sg_RNG();
 	else if(*gtype==10) this->sg_CCC();
+	else if(*gtype==11) this->sg_STIR();
 }
 
 /********************************************************************************************/
@@ -186,15 +192,16 @@ void Graph::sg_geometric(double *R)
 	if(*dbg)printf("Geometric (R=%f):",*R);
 	int i,j;
 	double dist;
-	for(i=0;i<(*pp->n-1);i++)
-		for(j=i+1;j<*pp->n;j++)
+	for(i=0;i<(pp->size()-1);i++)
+		for(j=i+1;j<pp->size();j++)
 		{
-			dist = Dist(&i,&j);
+			dist = pp->getDist(&i, &j);
 			if(dist<*R){
 				nodelist[i].push_back(j+1);
 				nodelist[j].push_back(i+1);
 			}
 		}
+	mdeg = this->pp->lambda*PI*(*R)*(*R);
 	if(*dbg)printf(" Ok.");
 }
 
@@ -204,20 +211,22 @@ void Graph::sg_shrink_geometric(double *R)
 	int i,j,j0;
 	double dist;
 	std::vector<int> *node;
-	for(i=0; i < *pp->n ; i++)
-	{
-		node = new std::vector<int>;
-		for(j=0;j < (int) this->nodelist[i].size() ; j++)
+	for(i=0; i < pp->size() ; i++)
+		if(inc[i])
 		{
-			j0 = nodelist[i][j]-1;
-			dist = Dist(&i,&j0);
-			if(dist<*R)
-				node->push_back(j0+1);
+			node = new std::vector<int>;
+			for(j=0;j < (int) this->nodelist[i].size() ; j++)
+			{
+				j0 = nodelist[i][j]-1;
+				dist = pp->getDist(&i,&j0);
+				if(dist<*R)
+					node->push_back(j0+1);
+			}
+			nodelist[i].clear();nodelist[i].resize(0);
+			for (j = 0; j < (int)node->size(); ++j) this->nodelist[i].push_back(node->at(j));
+			delete node;
 		}
-		nodelist[i].clear();nodelist[i].resize(0);
-		for (j = 0; j < (int)node->size(); ++j) this->nodelist[i].push_back(node->at(j));
-		delete node;
-	}
+	mdeg = this->pp->lambda*PI*(*R)*(*R);
 	if(*dbg)printf(" Ok.");
 }
 /********************************************************************************************/
@@ -226,13 +235,16 @@ void Graph::sg_mass_geometric()
 	if(*dbg)printf("Mass-geometric:");
 	int i,j;
 	double dist;
-	for(i=0;i<(*pp->n-1);i++)
-		for(j=i+1;j<*pp->n;j++)
+	for(i=0;i<pp->size();i++)
+		for(j=0;j<pp->size();j++)
 		{
-			dist = Dist(&i,&j);
-			if(dist< pp->mass[i]){
-				nodelist[i].push_back(j+1);
-				nodelist[j].push_back(i+1);
+			if(i!=j)
+			{
+				dist = pp->getDist(&i, &j);
+				if(dist< pp->getMass(&i)){
+					nodelist[i].push_back(j+1);
+	//				nodelist[j].push_back(i+1);
+				}
 			}
 		}
 	if(*dbg)printf(" Ok.");
@@ -241,21 +253,21 @@ void Graph::sg_mass_geometric()
 void Graph::sg_knn()
 {
 
-	int i,j,l,*k,kk;
+	int i,j,l,*k,kk, mik;
 	kk = (int) par[0];
 	k = &kk;
 	std::vector<int> *node;
 	if(*prepR==0)// if not preprocessed
 	{
 		if(*dbg)printf("%i-nn:",*k);
-		double *dists2_i = new double[*pp->n];
-		double *dists2_i2 = new double[*pp->n];
-		for(i=0;i<*pp->n;i++) //for each point
+		double *dists2_i = new double[pp->size()], *dists2_i2 = new double[pp->size()];
+		for(i=0;i<pp->size();i++) //for each point
+		if(inc[i])
 		{
-			for(j=0;j<*pp->n;j++) dists2_i2[j]=dists2_i[j]= Dist(&i,&j); //gather the distances to others
-			qsort( dists2_i, *pp->n, sizeof(double),compare_doubles); // sort distances, rising
+			for(j=0;j<pp->size();j++) dists2_i2[j]=dists2_i[j]= pp->getDist(&i, &j); //gather the distances to others
+			qsort( dists2_i, pp->size(), sizeof(double),compare_doubles); // sort distances, rising
 			for(j=1;j<=*k;j++) // find the k nearest
-				for(l=0;l<*pp->n;l++)
+				for(l=0;l<pp->size();l++)
 					if( dists2_i[j] == dists2_i2[l] ) //with distance comparison
 					{
 						nodelist[i].push_back(l+1);
@@ -267,37 +279,42 @@ void Graph::sg_knn()
 	else{ //preprocessed
 		if(*dbg)printf("%i-nn (shrinking):",*k);
 		double *dists2_i, *dists2_i2;
-		for(i=0;i<*pp->n;i++) //for each point
+		for(i=0;i<pp->size();i++) //for each point
+		if(inc[i])
 		{
 			node = new std::vector<int>;
 			dists2_i = new double [nodelist[i].size()];
 			dists2_i2 = new double [nodelist[i].size()];
-			if( (int) nodelist[i].size()<*k){ printf("\n preprocessing R too small, not enough neighbours (point #%i)!!\n",i+1);}
-			else// in case there is more than k precalc neighbours
+			mik = *k;
+			if( (int) nodelist[i].size()<*k )
 			{
-				for(l =	0;l < (int) nodelist[i].size();l++)
-				{
-					j = nodelist[i][l]-1;
-					dists2_i2[l]=Dist(&i,&j); //gather the distances to others, given preprocessing
-					dists2_i[l]=dists2_i2[l];
-				}
-				qsort( dists2_i, nodelist[i].size() , sizeof(double),compare_doubles); // sort distances, rising
-				for(j=0;j<*k;j++) // find the k nearest
-					for(l=0;l< (int) nodelist[i].size();l++)
-						if( dists2_i[j] == dists2_i2[l] ) //with distance comparison
-						{
-							node->push_back(nodelist[i][l]);
-							break;
-						}
-				nodelist[i].clear();nodelist[i].resize(0);
-				for(j=0;j < (int) node->size();j++) nodelist[i].push_back( (*node)[j] );
-				delete node;
-				delete[] dists2_i;
-				delete[] dists2_i2;
+				mik = nodelist[i].size();
+				printf("\n preprocessing R too small, not enough neighbours (point #%i)!!\n",i+1);
 			}
+			for(l=0;l< (int) nodelist[i].size();l++)
+			{
+				j = nodelist[i][l]-1;
+				dists2_i2[l]=pp->getDist(&i, &j); //gather the distances to others, given preprocessing
+				dists2_i[l]=dists2_i2[l];
+			}
+			qsort( dists2_i, nodelist[i].size() , sizeof(double),compare_doubles); // sort distances, rising
+
+			for(j=0;j< mik;j++) // find the k nearest
+				for(l=0;l< (int) nodelist[i].size();l++)
+					if( dists2_i[j] == dists2_i2[l] ) //with distance comparison
+					{
+						node->push_back(nodelist[i][l]);
+						break;
+					}
+			nodelist[i].clear();nodelist[i].resize(0);
+			for(j=0;j < (int) node->size();j++) nodelist[i].push_back( node->at(j) );
+			delete node;
+			delete[] dists2_i;
+			delete[] dists2_i2;
 		}
 	}
-	 if(*dbg)printf(" Ok.");
+	mdeg = kk;
+	if(*dbg)printf(" Ok.");
 }
 
 void Graph::sg_shrink_knn()
@@ -312,28 +329,28 @@ void Graph::sg_shrink_knn()
 void Graph::sg_gabriel()
 {
 	int kk = (int) par[0];
-	if(*dbg & kk>0)printf("%i-",kk);
+	if(*dbg & (kk>0) )printf("%i-",kk);
 	if(*dbg)printf("Gabriel:");
 	int i,j,k, empty,m,l,h;
 	double x0,y0,R2, d;
 	std::vector<int> *node;
 
 	if(*this->prepR == 0) // no preprocessing done,a heavy looping
-	  for(i=0;i<(*pp->n-1);i++)
+	  for(i=0;i<(pp->size()-1);i++)
 	  {
-		  for(j=i+1;j<*pp->n;j++)
+		  for(j=i+1;j<pp->size();j++)
 		  {
-			  x0 = fabs(pp->x[i]-pp->x[j])/2.0+fmin(pp->x[i],pp->x[j]);
-			  y0 = fabs(pp->y[i]-pp->y[j])/2.0+fmin(pp->y[i],pp->y[j]);
-			  R2 = ( pow(pp->x[i]-pp->x[j],2) + pow(pp->y[i]-pp->y[j],2) )/4.0;
+			  x0 = fabs(pp->getX(&i)-pp->getX(&j))/2.0+fmin(pp->getX(&i),pp->getX(&j));
+			  y0 = fabs(pp->getY(&i)-pp->getY(&j))/2.0+fmin(pp->getY(&i),pp->getY(&j));
+			  R2 = ( pow(pp->getX(&i)-pp->getX(&j),2) + pow(pp->getY(&i)-pp->getY(&j),2) )/4.0;
 			  //		brute force
 			  empty = 1+kk;
-			  for(k=0;k<*pp->n;k++)
+			  for(k=0;k<pp->size();k++)
 			  {
 				  if(k != i)
 					  if( k != j)
 					  {
-						  d = pow(x0-pp->x[k],2) + pow(y0-pp->y[k],2);
+						  d = pow(x0-pp->getX(&k),2) + pow(y0-pp->getY(&k),2);
 						  if( d<R2 )
 						  {
 							  empty = empty - 1;
@@ -349,23 +366,23 @@ void Graph::sg_gabriel()
 	  }
 	else{ // preprocessed: nodelist has the restricted neighbourhoods to look trough
 		if(*dbg)printf("(prepd): ");
-		for(i = 0 ; i< *pp->n ;  i++)
+		for(i = 0 ; i< pp->size() ;  i++)
 		{
 			node = new std::vector<int>;
 			for( l=0 ; l < (int) this->nodelist[i].size(); l++ )
 			{
 				j = this->nodelist[i][l]-1;
-				x0 = fabs(this->pp->x[i]-this->pp->x[j])/2.0+fmin(this->pp->x[i],this->pp->x[j]);
-				y0 = fabs(this->pp->y[i]-this->pp->y[j])/2.0+fmin(this->pp->y[i],this->pp->y[j]);
-				R2 = (pow(this->pp->x[i]-this->pp->x[j],2) + pow(this->pp->y[i]-this->pp->y[j],2) )/4.0;
+				x0 = fabs(this->pp->getX(&i)-this->pp->getX(&j))/2.0+fmin(this->pp->getX(&i),this->pp->getX(&j));
+				y0 = fabs(this->pp->getY(&i)-this->pp->getY(&j))/2.0+fmin(this->pp->getY(&i),this->pp->getY(&j));
+				R2 = (pow(this->pp->getX(&i)-this->pp->getX(&j),2) + pow(this->pp->getY(&i)-this->pp->getY(&j),2) )/4.0;
 				empty = 1+kk;
-				for(m=0; m < (int)this->nodelist[i].size();m++) // the small ball is included in the preprocessing ball
+				for(m=0; m <(int) this->nodelist[i].size();m++) // the small ball is included in the preprocessing ball
 				{
 					k =  (int) this->nodelist[i][m]-1;
 					if(k != i)
 						if( k != j)
 						{
-							d = pow(x0-pp->x[k],2) + pow(y0-pp->y[k],2);
+							d = pow(x0-pp->getX(&k),2) + pow(y0-pp->getY(&k),2);
 							if( d<R2 )
 							{
 								empty = empty - 1;
@@ -383,26 +400,26 @@ void Graph::sg_gabriel()
 			delete node;
 		}
 	}
-	  if(*dbg)printf(" Ok.");
+	mdeg = 4;
+	if(*dbg)printf(" Ok.");
 }
 
 /********************************************************************************************/
-void Graph::sg_delauney()
+void Graph::sg_delaunay()
 {
 //Naive algorithm, checks the interiors of triangle circumcircles.
 //For 2D patterns
 
-	if(*dbg)printf("Delauney: ");
+	if(*dbg)printf("Delaunay: ");
 	int i,j,k,l,h;
-	double dummy[2];
 	std::vector<int> *node;
 	if(*this->prepR==0) // no preprocessing done, heavy looping
 	{
 		if(*dbg)printf("(raw):");
-		for(i = 0 ; i< *pp->n-2 ; i++ )
-			for(j = i+1 ; j < *pp->n-1 ; j++ )
-				for(k = j+1 ; k < *pp->n ; k++ )
-					if( Empty(pp->x,pp->y,pp->n,i,j,k, dummy, dummy, dummy) )
+		for(i = 0 ; i< pp->size()-2 ; i++ )
+			for(j = i+1 ; j < pp->size()-1 ; j++ )
+				for(k = j+1 ; k < pp->size() ; k++ )
+					if( pp->Empty(&i,&j,&k) )
 					{
 						addNew(i,j+1); addNew(i,k+1);
 						addNew(j,i+1); addNew(j,k+1);
@@ -411,7 +428,7 @@ void Graph::sg_delauney()
 	}
 	else{ // preprocessed: nodelist has the restricted neighbourhoods to look trough for triangles
 		if(*dbg)printf("(prepd): ");
-		for(i = 0 ; i< *pp->n ;  i++)
+		for(i = 0 ; i< pp->size() ;  i++)
 		{
 			node = new std::vector<int>;
 			for( l=0 ; l < (int)nodelist[i].size()-1; l++ )
@@ -420,7 +437,7 @@ void Graph::sg_delauney()
 				for(h=l+1; h < (int)nodelist[i].size();h++ )
 				{
 					k = nodelist[i][h]-1;
-					if( Empty(pp->x,pp->y,pp->n,i,j,k,dummy, dummy, dummy) )
+					if( pp->Empty(&i, &j, &k) )
 					{
 						node->push_back(j+1);
 						node->push_back(k+1);
@@ -433,6 +450,7 @@ void Graph::sg_delauney()
 		}
 		this->remove_duplicates();
 	}
+	mdeg = 6;
 	if(*dbg)printf(" Ok.");
 
 }
@@ -441,17 +459,15 @@ void Graph::sg_MST()
 {
   if(*this->dbg) printf("MST:");
   int i,j,k=0,l=0,zz,k0=0,l0=0;
-  int *done = new int[*this->pp->n], dn;
+  int *done = new int[pp->size()],dn;
   double apu0,apu1,apu2;
   done[0] = 0;
   dn = 1;
-  int left=*this->pp->n-dn;
-//   double sum;
+  int left=pp->size()-dn;
   while( left > 0 )
   {
-    //if(*print_sg) if((left+1)%100==0) printf("MST: %i/%i         \r",left,*n);
     apu2 = MAX_DOUBLE;
-    for(i=1; i<*this->pp->n;i++){
+    for(i=1; i<pp->size();i++){
       zz = 1;
       apu0=apu2;
       for(j=0; j<dn;j++){
@@ -459,7 +475,7 @@ void Graph::sg_MST()
           zz=0;
           break;
         }
-        apu1 = Dist(&i,&done[j]); //dists[i*(*n)+done[j]];
+        apu1 = pp->getDist(&i,&done[j]);
         if( apu1<apu0 ){
           apu0=apu1;
           k0=i;
@@ -474,13 +490,10 @@ void Graph::sg_MST()
         }
       }
     }
-//     sum+=apu2;
     done[dn] = k;
     dn++;
     left--;
     this->nodelist[l].push_back(k+1);
-    //e[l*(*n)+k] = 1;
-//     printf("\r");
   }
   if(*this->dbg)printf(" Ok.");
 }
@@ -490,11 +503,11 @@ void Graph::sg_markcross()
 	if(*dbg)printf("Markcross: ");
 	int i,j;
 	double dist;
-	for(i=0;i<(*pp->n-1);i++)
-		for(j=i+1;j<*pp->n;j++)
+	for(i=0;i<(pp->size()-1);i++)
+		for(j=i+1;j<pp->size();j++)
 		{
-			dist = Dist(&i,&j);
-			if(dist< pp->mass[i]+pp->mass[j]){
+			dist = pp->getDist(&i, &j);
+			if(dist< pp->getMass(&i)+pp->getMass(&j)){
 				nodelist[i].push_back(j+1);
 				nodelist[j].push_back(i+1);
 			}
@@ -507,12 +520,12 @@ void Graph::sg_SIG()
 	if(*dbg)printf("Spheres-of-Influence:");
 	int i,j,dbg0=*dbg;
 	double dist;
-	for(i=0;i<*pp->n;i++)
+	for(i=0;i<pp->size();i++)
 	{
 		dist = MAX_DOUBLE;
-		for(j=0;j<*pp->n;j++)
-			if(i!=j) dist = fmin(dist, Dist(&i,&j));
-		pp->mass[i]=dist;
+		for(j=0;j<pp->size();j++)
+			if(i!=j) dist = fmin(dist, pp->getDist(&i, &j));
+		pp->setMass(&i,dist);
 	}
 	*dbg=0;
 	sg_markcross();
@@ -523,23 +536,23 @@ void Graph::sg_SIG()
 void Graph::sg_RST()
 {
   if(*dbg) printf("Radial Spanning Tree (o=(%f,%f,%f)): ",par[0],par[1],par[2]);
-  nodelist.resize(*pp->n-1);
-  int i,j,k,foc_i=*pp->n-1;
+  nodelist.resize(pp->size()-1);
+  int i,j,k,foc_i=pp->size()-1;
   double apu0,apu1,apu2,apu3;
 
-  for(i=0;i<*pp->n-1;i++)
+  for(i=0;i<pp->size()-1;i++)
   {
-    apu0 = Dist(&i,&foc_i);//dists[i*(*n+1)+*n];
+    apu0 = pp->getDist(&i,&foc_i);//dists[i*(*n+1)+*n];
     apu3=MAX_DOUBLE;
     k=-1;
-    for(j=0;j<*pp->n-1;j++)
+    for(j=0;j<pp->size()-1;j++)
     {
       if(j!=i)
       {
-        apu1 = Dist(&j,&foc_i);//dists[j*(*n+1)+*n];
+        apu1 = pp->getDist(&j,&foc_i);//dists[j*(*n+1)+*n];
         if(apu1 < apu0 )
         {
-          apu2 = Dist(&i,&j);//dists[i*(*n+1)+j];
+          apu2 = pp->getDist(&i, &j);//dists[i*(*n+1)+j];
           if( apu2 < apu3 )
           {
             apu3 = apu2;
@@ -557,15 +570,15 @@ void Graph::sg_RNG()
 {
 	if(*dbg) printf("Relative neighbourhood: ");
 	int i,j,k,isempty;
-    for(i=0;i<(*pp->n-1);i++)
+    for(i=0;i<(pp->size()-1);i++)
     {
-        for(j=i+1;j<*pp->n;j++)
+        for(j=i+1;j<pp->size();j++)
         {
         	isempty = 1;
-        	for(k=0;k<*pp->n;k++)
-        		if(k!=i&k!=j)
-        			if(Dist(&i,&k) < Dist(&i,&j))
-        				if(Dist(&j,&k) < Dist(&j,&i))
+        	for(k=0;k<pp->size();k++)
+        		if( (k!=i) & (k!=j) )
+        			if(pp->getDist(&i,&k) < pp->getDist(&i, &j))
+        				if(pp->getDist(&j,&k) < pp->getDist(&j,&i))
         				{isempty=0;break;}
         	if(isempty)
         	{
@@ -579,27 +592,69 @@ void Graph::sg_RNG()
 /********************************************************************************************/
 void Graph::sg_CCC()
 {
+	double m=MAX_DOUBLE, mm = -MAX_DOUBLE, apu;
 	if(*dbg) printf("Class Cover Catch for type=%i: ",(int)par[0]);
 	int i,j, type0=(int)par[0];
-	for(i=0; i<*pp->n;i++)
+	for(i=0; i<pp->size();i++)
 	{
-		pp->mass[i]=-MAX_DOUBLE;
-		if(pp->type[i]==type0)
+		pp->setMass(&i,mm);
+		if(pp->getT(&i)==type0)
 		{
-			pp->mass[i]=MAX_DOUBLE;
-			for(j=0;j<*pp->n;j++)
-				if(j!=i & pp->type[j]!=type0) pp->mass[i]=fmin(pp->mass[i],Dist(&i,&j));
+			pp->setMass(&i, m);
+			for(j=0;j<pp->size();j++)
+				if( (j!=i) & (pp->getT(&j)!=type0) )
+				{
+					apu = fmin(pp->getMass(&i),pp->getDist(&i, &j));
+					pp->setMass(&i, apu);
+				}
 		}
 	}
-	for(i=0;i<*pp->n;i++) //TODO: optimize this
-		if(pp->type[i]==type0)
-			for(j=0;j<*pp->n;j++)
+	for(i=0;i<pp->size();i++) //TODO: optimize this
+		if(pp->getT(&i)==type0)
+			for(j=0;j<pp->size();j++)
 				if(i!=j)
-					if(pp->type[j]==type0)
-						if(Dist(&i,&j)< pp->mass[i])
+					if(pp->getT(&j)==type0)
+						if(pp->getDist(&i, &j)< pp->getMass(&i))
 							addNew(i,j+1);
 	if(*dbg) printf(" Ok.");
 }
+/********************************************************************************************/
+double Attenuate(double r, double alpha)
+{
+	return pow(1.0+r,-alpha);
+}
+
+void Graph::sg_STIR()
+{
+	if(*dbg) printf("Signal-To-Noise-Ratio graph, noise=%f,alpha=%f,beta=%f,gamma=%f: ",par[0],par[1],par[2],par[3]);
+	int i,j;
+	double noise0 = par[0], alpha=par[1],beta=par[2],gamma=par[3], *noise = new double[pp->size()], sij,sji,s;
+	//first we compute the interference for each location including the sending transmitter
+	for(i=0;i<pp->size();i++)
+	{
+		noise[i]=0.0;
+		for(j=0;j<pp->size();j++)
+			if(i!=j) noise[i] = noise[i]+ pp->getMass(&j)*Attenuate(pp->getDist(&i, &j),alpha);
+	}
+
+	for(i=0;i<(pp->size()-1);i++)
+		for(j=i+1;j<pp->size();j++)
+		{
+			sij = pp->getMass(&i)*Attenuate(pp->getDist(&i, &j),alpha);
+			sji = pp->getMass(&j)*Attenuate(pp->getDist(&i, &j),alpha);
+			sij = sij/(noise0 + gamma* (noise[j]-sij));
+			sij = sji/(noise0 + gamma* (noise[i]-sji));
+			s = fmin(sij,sji);
+			if(s >= beta )
+			{
+				addNew(i,j+1);
+				addNew(j,i+1);
+			}
+		}
+	if(*dbg) printf(" Ok.");
+}
+
+
 /********************************************************************************************/
 // cut (remove) the graph edges longer than R
 void Graph::sg_cut(double *R)
@@ -607,14 +662,14 @@ void Graph::sg_cut(double *R)
 	int i,j,k, count=0;
 	if(*dbg)printf("Cutting the graph (R=%f):",*R);
 	std::vector<int > *pnode;
-	for(i=0;i < *pp->n;i++)
+	for(i=0;i < pp->size();i++)
 	{
 		pnode = new std::vector<int>;
 		pnode->resize(0);
 		for(j=0; j < (int)nodelist.at(i).size();j++)
 		{
 			k = nodelist.at(i).at(j)-1;
-			if( Dist(&i, &k) < *R )
+			if( pp->getDist(&i, &k) < *R )
 				pnode->push_back(k+1);
 			else
 				count++;
@@ -688,4 +743,15 @@ void Graph::sg_prune(double *lev)
 	if(*dbg)printf(" Ok (%i branches pruned).",count);
 }
 /********************************************************************************************/
+
+
+/**********************************************************************************/
+int compare_doubles(const void *a, const void *b)
+{
+  const double *da = (const double *) a;
+  const double *db = (const double *) b;
+
+  return (*da > *db) - (*da < *db);
+}
+
 // EOF
